'use server';

import { requestGetFetch, requestPostFetch } from "@/utils/fetch";
import { getServerSession, Session } from "next-auth";
import { nextauthOptions } from "../nextauth-options";

export async function getKycs(searchParams?: { [key: string]: string | number }) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    
    // Dynamically build the query string from searchParams
    const paramsArray = searchParams && Object.entries(searchParams);
    const queryString = paramsArray
      ? paramsArray
          .map(([key, value]) => `${key}=${encodeURIComponent(String(value))}`)
          .join('&')
      : '';

    const path = 'v1/kycs' + (queryString ? `?${queryString}` : '');

    const response = await requestGetFetch({
      path,
      options: {
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        cache: "no-store",
      },
    });

    return response.data || [];
  } catch (error) {
    throw error;
  }
}

export async function viewKyc(id: number) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const path = `v1/kycs/${id}`;
    const response = await requestGetFetch({
      path,
      options: {
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        cache: "no-store",
      },
    });
    return response.data || [];
  } catch (error) {
    throw error;
  }
}


export async function getKycById(id: number) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const path = `v1/kycs/${id}`;
    const response = await requestGetFetch({
      path,
      options: {
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        cache: "no-store",
      },
    });
    return response.data || null;
  } catch (error) {
    throw error;
  }
}

export async function deleteKyc(id: number) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const path = `v1/kycs/${id}`;
    const response = await requestGetFetch({
      path,
      options: {
        method: "DELETE",
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        cache: 'no-store',
      },
    });
    return response;
  } catch (error) {
    throw error;
  }
}

interface CreateKycProps {
  name: string;
  permissions: string[];
}

export async function createKyc({
  name,
  permissions,
}: CreateKycProps) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const response = await requestPostFetch({
      path: `v1/kycs`,
      data: {
        name,
        permissions,
      },
      token: token,
    });
    return response;
  } catch (error: any) {
    throw new Error(error.message);
  }
}

interface UpdateKycProps {
  id: number;
  name: string;
  permissions: string[];
}

export async function updateKyc({
  id,
  name,
  permissions,
}: UpdateKycProps) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const path = `v1/kycs/${id}`;

    const response = await requestGetFetch({
      path,
      options: {
        method: "PUT",
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        body: JSON.stringify({
          name,
          permissions,
        }),
        cache: 'no-store',
      },
    });
    return response;
  } catch (error: any) {
    throw new Error(error.message);
  }
}

export async function createFileRequest(payload: any) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user?.token;

    const path = "v1/files";

    const response = await requestPostFetch({
      path,
      data: payload,
      token: token,
    });

    return response
  } catch (error: any) {
    throw new Error(error.message);
  }
}

export async function createKycRequest(payload: any) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user?.token;

    const path = "v1/kycs";

    const response = await requestPostFetch({
      path,
      data: { ...payload },
      token: token,
    });

    return response
  } catch (error: any) {
    throw new Error(error.message);
  }
}